#!/usr/bin/perl
use strict;
#
# Script to create a bootable sys$rom.bin image for Psion 5mx Pro
# Copyright 2002 Tony Lindgren <tony@atomide.com>
#

# Boot loader code start offset
my $BOOTSTART = 0xf3c;

my $IMAGE = "sys\$rom.bin";

my $BOOTBIN = "proboot.bin";
my $LINUX_TEXTADDR = 0xc0048000;
my $LINUXSTART = (($LINUX_TEXTADDR << 2) >> 2);

#my $LINUXIMAGE = "/home/arm/dev/linux/arch/arm/boot/Image";
my $LINUXIMAGE = "";
my $LINUXPARAMS = "";
my $LINUXINITRD = "";

#
# Main program
#
($LINUXIMAGE, $LINUXPARAMS, $LINUXINITRD) = @ARGV;

if ($LINUXIMAGE eq "") {
  print "\nUsage: ".$0." linuximage params initrd\n\n";
  exit;
}

print "Using kernel image: ".$LINUXIMAGE."\n";
print "Using kernel parameters: ".$LINUXPARAMS."\n";
print "Using initrd: ".$LINUXINITRD."\n";

#
# First we need to create a 256 byte header for the
# image that works with the Psion boot loader. We
# only need one ARM instruction there to tell where
# to jump to start the proboot execution.
#
create_header($IMAGE);
my $imagesize = -s $IMAGE;
pad_file($IMAGE, ($BOOTSTART - $imagesize), 0);

#
# Append the proboot binary to the image
#
concat_files($IMAGE, $BOOTBIN);

#
# Pad until we are at the Linux TEXTADDR, and then append
# the Linux kernel image
#
$imagesize = -s $IMAGE;
pad_file($IMAGE, ($LINUXSTART - $imagesize));
concat_files($IMAGE, $LINUXIMAGE, 0);

print "\nBoot image creation done. The new boot image is ".$IMAGE."\n\n";

exit;

#
# Subroutines
#
sub create_header() {
	my($file) = @_;
	open OUT, ">$file"
		or die "ERROR: Cannot open file ".$file;
	print "Writing boot image header\n";
	print OUT "\0\0\0\0";
	# Write out some ARM assembly: cc 03 00 ea
	print OUT "\xcc\x03\x00\xea";
	close (OUT);
}

sub concat_files() {
	my($file1, $file2) = @_;
	open OUT, ">>$file1"
		or die "ERROR: Cannot open file ".$file1;
	open IN, "<$file2"
		or die "ERROR: Cannot open file ".$file2;
	print "Appending ".$file2." to ".$file1."\n";
	while (<IN>) {
		print OUT;
	}
	close (IN);
	close (OUT);
	my $concat_size = -s $file1;
	print "The new size is: ".$concat_size."\n";
}

sub pad_file() {
	my($file, $padsize, $overwrite) = @_;
	if ($overwrite gt 0) {
		print"Overwriting ".$file."\n";
		open OUT, ">$file";
	} else {
		open OUT, ">>$file";
	}

	print "Padding ".$file." with ".$padsize." bytes\n";
	for (my $i=0; $i < $padsize; $i++) {
		print OUT "\0";
	}
	close (OUT);
	my $padded_size = -s $file;
	print "The new size is: ".$padded_size."\n";
}
